"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Token_1 = __importStar(require("./Token"));
class Scanner {
    source;
    tokens = [];
    start = 0;
    current = 0;
    line = 1;
    keywords = {
        "@else": Token_1.TokenType.ELSE,
        "@for": Token_1.TokenType.FOR,
        "@if": Token_1.TokenType.IF,
        "@while": Token_1.TokenType.WHILE,
        "false": Token_1.TokenType.FALSE,
        "null": Token_1.TokenType.NULL,
        "undefined": Token_1.TokenType.UNDEFINED,
        "true": Token_1.TokenType.TRUE,
    };
    constructor(source) {
        this.source = source;
    }
    scanTokens() {
        while (!this.isAtEnd()) {
            this.start = this.current;
            this.scanToken();
        }
        this.tokens.push(new Token_1.default(Token_1.TokenType.EOF, "", null, this.line));
        return this.tokens;
    }
    isAtEnd() {
        console.info(this.current, this.source.length);
        return this.current >= this.source.length;
    }
    scanToken() {
        const c = this.advance();
        switch (c) {
            case '(':
                this.addToken(Token_1.TokenType.RIGHT_PAREN);
                break;
            case ')':
                this.addToken(Token_1.TokenType.LEFT_PAREN);
                break;
            case '{':
                this.addToken(Token_1.TokenType.RIGHT_BRACE);
                break;
            case '}':
                this.addToken(Token_1.TokenType.LEFT_BRACE);
                break;
            case ',':
                this.addToken(Token_1.TokenType.COMMA);
                break;
            case '.':
                this.addToken(Token_1.TokenType.DOT);
                break;
            case '-':
                this.addToken(Token_1.TokenType.MINUS);
                break;
            case '+':
                this.addToken(Token_1.TokenType.PLUS);
                break;
            case ';':
                this.addToken(Token_1.TokenType.SEMICOLON);
                break;
            case '*':
                this.addToken(Token_1.TokenType.STAR);
                break;
            case '!':
                if (this.match('=')) {
                    this.addToken(this.match('=') ? Token_1.TokenType.BANG_EQUAL_EQUAL : Token_1.TokenType.BANG_EQUAL);
                }
                else {
                    this.addToken(Token_1.TokenType.BANG);
                }
                break;
            case '=':
                if (this.match('=')) {
                    this.addToken(this.match('=') ? Token_1.TokenType.EQUAL_EQUAL_EQUAL : Token_1.TokenType.EQUAL_EQUAL);
                }
                else {
                    this.addToken(Token_1.TokenType.EQUAL);
                }
                break;
            case '<':
                this.addToken(this.match('=') ? Token_1.TokenType.LESS_EQUAL : Token_1.TokenType.LESS);
                break;
            case '>':
                this.addToken(this.match('=') ? Token_1.TokenType.GREATER_EQUAL : Token_1.TokenType.GREATER);
                break;
            case ' ':
            case '\r':
            case '\t':
                // Ignore whitespace.
                break;
            case '\n':
                this.line++;
                break;
            case '"':
                this.string();
                break;
            default:
                if (this.isDigit(c)) {
                    this.number();
                }
                else if (this.isAlpha(c)) {
                    this.identifier();
                }
                else {
                    console.error(this.line, "Unexpected character.");
                }
                break;
        }
    }
    advance() {
        return this.source.charAt(this.current++);
    }
    addToken(type, literal) {
        const text = this.source.substring(this.start, this.current);
        this.tokens.push(new Token_1.default(type, text, literal, this.line));
    }
    match(expected) {
        if (this.isAtEnd())
            return false;
        if (this.source.charAt(this.current) != expected)
            return false;
        this.current++;
        return true;
    }
    peek() {
        if (this.isAtEnd())
            return '\0';
        return this.source.charAt(this.current);
    }
    peekNext() {
        if (this.current + 1 >= this.source.length)
            return '\0';
        return this.source.charAt(this.current + 1);
    }
    string() {
        while (this.peek() != '"' && !this.isAtEnd()) {
            if (this.peek() === '\n') {
                this.line++;
            }
            this.advance();
        }
        if (this.isAtEnd()) {
            console.error();
            throw new Error('Unterminated string on line ' + this.line + '.');
        }
        this.advance();
        const value = this.source.substring(this.start + 1, this.current - 1);
        this.addToken(Token_1.TokenType.STRING, value);
    }
    isDigit(char) {
        return char >= '0' && char <= '9';
    }
    isAlpha(char) {
        return (char >= 'a' && char <= 'z') ||
            (char >= 'A' && char <= 'Z') ||
            char === '_';
    }
    isAlphaNumeric(char) {
        return this.isAlpha(char) || this.isDigit(char);
    }
    number() {
        while (this.isDigit(this.peek()))
            this.advance();
        // Look for a fractional part
        if (this.peek() === '.' && this.isDigit(this.peekNext())) {
            // Consume the '.'
            this.advance();
            while (this.isDigit(this.peek()))
                this.advance();
        }
        this.addToken(Token_1.TokenType.NUMBER, parseFloat(this.source.substring(this.start, this.current)));
    }
    identifier() {
        while (this.isAlphaNumeric(this.peek()))
            this.advance();
        const text = this.source.substring(this.start, this.current);
        let type = this.keywords[text];
        if (type === undefined) {
            type = Token_1.TokenType.IDENTIFIER;
        }
        this.addToken(type);
    }
}
exports.default = Scanner;
