"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.view = exports.compile = void 0;
const Path_1 = __importDefault(require("../Path"));
const fs = require('fs');
const lib_1 = require("@rooo/views/lib");
const lib_2 = require("@rooo/views/lib");
const compiler = new lib_1.Compiler();
compiler.addDefaults();
const compile = (filepath) => {
    let compiled;
    try {
        const contents = fs.readFileSync(Path_1.default.viewPath(filepath), 'utf8');
        const source = new lib_1.Source(contents);
        compiled = compiler.compile(source);
    }
    catch (e) {
        if (e instanceof lib_1.AbstractError) {
            process.stdout.write(e.prettyPrint());
            process.exit(-1);
        }
        throw e;
    }
    return compiled;
};
exports.compile = compile;
const templateProvider = {
    async getTemplateCompiledFile(filepath) {
        return (new Function('__env', 'return ' + exports.compile(filepath) + '(__env)'));
    },
    async isOutdated(filepath, template, creationTime) {
        const stat = await fs.stat(filepath);
        return stat.mtimeMs > creationTime;
    },
};
const view = async (filepath, params = {}) => {
    let compiledView = '';
    const runtime = new lib_2.Runtime([templateProvider]);
    for await (const chunk of runtime.render(filepath, params)) {
        compiledView += chunk.toString();
    }
    return compiledView;
};
exports.view = view;
